﻿#pragma once

#include <vector>

#include "DefineDllForWin.h"

namespace CLOAPI
{
	class MV_CLO_INTERFACE_API FabricAPIInterface {
	public:
		/// @fn GetFabricCount()
		/// @brief Get the number of fabrics
		/// @return the number of fabric in object browser for the current project.
		virtual unsigned int GetFabricCount() { return 0; }

		/// @fn GetCurrentFabricIndex()
		/// @brief Get the index of the selected Fabric
		/// @return the index of selected fabric in object browser for the current project..
		virtual unsigned int GetCurrentFabricIndex() { return 0; }

		/// @fn ExportZFab()
		/// @brief Export ZFab which cotains the fabric data selected in the object browser
		/// @return Output file path;output files will be created in CLO temporary folder. If an error occurs, return empty string.
		virtual std::string ExportZFab() { return ""; }

		/// @fn ExportZFab(const string& filePath)
		/// @brief Export Zfab which cotains the fabric data selected in the object browser
		/// @param filePath: output file path
		/// @return Output file path. If an error occurs, return empty string. 
		virtual std::string ExportZFab(const std::string& filePath) { return ""; }

		/// @fn ExportZFab(const string& filePath, const int& index)
		/// @brief Export ZFab which cotains the fabric data in the index of the object browser
		/// @param filePath: output file path
		/// @param index: target fabric index on the object browser to export
		/// @return Output file path. If an error occurs, return empty string. 
		virtual std::string ExportZFab(const std::string& filePath, const int& index) { return ""; }

		/// @fn ExportZFabW(const wstring& filePath, const int& index)
		/// @brief Export ZFab file which cotains the fabric data in the index of the object browser
		/// @param filePath: output file path
		/// @param index: target fabric index on the object browser to export
		/// @return Output file path. If an error occurs, return empty string. 
		virtual std::wstring ExportZFabW(const std::wstring& filePath, const int& index) { return L""; }
		
		/// @fn ExportFabric(const string& filePath)
		/// @brief Export the selected fabric to the file(.jfab or .zfab)		
		/// @param filePath: output file path		
		/// @return Output file path. If an error occurs, return empty string.
		virtual std::string ExportFabric(const std::string& filePath) { return ""; }

		/// @fn ExportFabric(const string& filePath, const int& index)
		/// @brief Export the selected fabric to the file(.jfab or .zfab)		
		/// @param filePath: output file path		
		/// @param index: target fabric index on the object browser to export
		/// @return Output file path. If an error occurs, return empty string.
		virtual std::string ExportFabric(const std::string& filePath, const int& index) { return ""; }

		/// @fn ExportFabricW(const wstring& filePath, const int& index)
		/// @brief Export the selected fabric to the file(.jfab or .zfab)		
		/// @param filePath: output file path		
		/// @param index: target fabric index on the object browser to export
		/// @return Output file path. If an error occurs, return empty string.
		virtual std::wstring ExportFabricW(const std::wstring& filePath, const int& index) { return L""; }

		/// @fn AddFabric(const string& inputFilePath)
		/// @brief Add Fabric into the Object Browser
		/// @param inputFilePath: 'zfab'(clo file) path or 'jfab'(json) path
		/// @return the index of added fabric in object browser for the current project. .
		virtual unsigned int AddFabric(const std::string& inputFilePath) { return 0; }

		/// @fn AddFabricW(const wstring& inputFilePath)
		/// @brief Add Fabric into the Object Browser
		/// @param inputFilePath: 'zfab'(clo file) path or 'jfab'(json) path
		/// @return the index of added fabric in object browser for the current project. .
		virtual unsigned int AddFabricW(const std::wstring& inputFilePath) { return 0; }

		/// @fn GetFirstFabricTextureName()
		/// @brief Get the main image file name for the first fabric in the object browser for the current colorway index
		/// @return the name of the image file name without folder path and extension.
		virtual std::string GetFirstFabricTextureName() { return ""; }

		/// @fn GetFirstFabricTextureNameW()
		/// @brief Get the main image file name for the first fabric in the object browser for the current colorway index
		/// @return the name of the image file name without folder path and extension.
		virtual std::wstring GetFirstFabricTextureNameW() { return L""; }

		/// @fn GetFirstFabricTextureName(unsigned int colorwayIndex)
		/// @brief Get the main image file name for the first fabric in the object browser for a colorway index
		/// @param colorwayIndex: colorway index
		/// @return the name of the image file name without folder path and extension.
		virtual std::string GetFirstFabricTextureName(unsigned int colorwayIndex) { return ""; }

		/// @fn GetFirstFabricTextureNameW(unsigned int colorwayIndex)
		/// @brief Get the main image file name for the first fabric in the object browser for a colorway index
		/// @param colorwayIndex: colorway index
		/// @return the name of the image file name without folder path and extension.
		virtual std::wstring GetFirstFabricTextureNameW(unsigned int colorwayIndex) { return L""; }

		/// @fn ChangeFabricWithJson(unsigned int fabricIndex, const string& inputJsonFilePath)
		/// @brief Overwrite all the properties of a fabric with json file(.jfab) 
		/// @param fabricIndex: target fabric index in the object browser
		/// @param inputJsonFilePath: the jfab file path to load on to the fabric in the fabricIndex
		/// @return if it succeeds, return true.
		virtual bool ChangeFabricWithJson(unsigned int fabricIndex, const std::string& inputJsonFilePath) { return false; }

		/// @fn ChangeFabricWithJsonW(unsigned int fabricIndex, const wstring& inputJsonFilePath)
		/// @brief Overwrite all the properties of a fabric with json file(.jfab) 
		/// @param fabricIndex: target fabric index in the object browser
		/// @param inputJsonFilePath: the jfab file path to load on to the fabric in the fabricIndex
		/// @return if it succeeds, return true.
		virtual bool ChangeFabricWithJsonW(unsigned int fabricIndex, const std::wstring& inputJsonFilePath) { return false; }

		/// @fn AssignFabricToPattern(unsigned int fabricIndex, unsigned int patternIndex)
		/// @brief Assign a fabric to a pattern. 
		/// @param fabricIndex: the source fabric index in the object browser to apply to the target pattern
		/// @param patternIndex: the tagert pattern index to apply the fabric in the fabricIndex
		/// @return if it succeeds, return true.
		virtual bool AssignFabricToPattern(unsigned int fabricIndex, unsigned int patternIndex) { return false; }	

		/// @fn ImportXtexFile(unsigned int fabricIndex, const string& xTexFilePath)		
		/// @brief Import xtex to a fabric. Overwrite all the values described in the xtex file into the Fabric
		/// @parm fabricIndex: the target fabric index in the object browser to import xTex file onto
		/// @parm xTexFilePath: the xTex file path to import
		/// @return if it succeeds, return true.
		virtual bool ImportXtexFile(unsigned int fabricIndex, const std::string& xTexFilePath) { return false; }

		/// @fn ImportXtexFileW(unsigned int fabricIndex, const wstring& xTexFilePath)
		/// @brief Import xtex to a fabric. Overwrite all the values described in the xtex file into the Fabric
		/// @parm fabricIndex: the target fabric index in the object browser to import xTex file onto
		/// @parm xTexFilePath: the xTex file path to import
		/// @return if it succeeds, return true.
		virtual bool ImportXtexFileW(unsigned int fabricIndex, const std::wstring& xTexFilePath) { return false; }

		/// @fn ApplyXtexFile(unsigned int fabricIndex, const string& xTexFilePath)
		/// @brief Apply xtex to a fabric. Only set the values described in the xtex file into the Fabric.
		/// @parm fabricIndex: the target fabric index in the object browser to apply xTex file onto
		/// @parm xTexFilePath: the xTex file path to apply
		/// @return if it succeeds, return true.
		virtual bool ApplyXtexFile(unsigned int fabricIndex, const std::string& xTexFilePath) { return false; }

		/// @fn ApplyXtexFile(unsigned int fabricIndex, const string& xTexFilePath)
		/// @brief Apply xtex to a fabric. Only set the values described in the xtex file into the Fabric.
		/// @parm fabricIndex: the target fabric index in the object browser to apply xTex file onto
		/// @parm xTexFilePath: the xTex file path to apply
		/// @return if it succeeds, return true.
		virtual bool ApplyXtexFileW(unsigned int fabricIndex, const std::wstring& xTexFilePath) { return false; }
		
		/// @fn DeleteFabric(unsigned int fabricIndex)
		/// @brief Delete a fabric. If the fabric was assigned on Patterns, the patterns will have the default fabric
		/// @return if it succeeds, return true.
		virtual bool DeleteFabric(unsigned int fabricIndex) { return false; }

		/// @fn GetAPIMetaData(unsigned int fabricIndex)
		/// @brief Get API meta data for the fabric
		/// @parm fabricIndex: target fabric index on the Object Browser
		/// @return json string for meta data [key - value] list
		virtual std::string GetAPIMetaData(unsigned int fabricIndex) { return "";		}

		/// @fn GetAPIMetaDataW(unsigned int fabricIndex)
		/// @brief Get API meta data for the fabric
		/// @parm fabricIndex: target fabric index on the Object Browser
		/// @return json string for meta data [key - value] list
		virtual std::wstring GetAPIMetaDataW(unsigned int fabricIndex) { return L""; }

		/// @fn SetAPIMetaData(unsigned int fabricIndex, string jsonStr)
		/// @brief Set API meta data for the fabric
		/// @param fabricIndex: target fabric index on the Object Browser
		/// @param jsonStr: api meta data in string format 
		/// @return if it succeeds, return true
		virtual bool SetAPIMetaData(unsigned int fabricIndex, std::string jsonStr) { return ""; }

		/// @fn SetAPIMetaData(unsigned int fabricIndex, string jsonStr)
		/// @brief Set API meta data for the fabric
		/// @param fabricIndex: target fabric index on the Object Browser
		/// @param jsonStr: api meta data in string format 
		/// @return if it succeeds, return true
		virtual bool SetAPIMetaDataW(unsigned int fabricIndex, std::wstring jsonStr) { return L""; }

		/// @fn ChangeMetaDataValueForFabric(unsigned int fabricIndex, const string& metaDataKey, const string& metaDataValue)
		/// @brief Change or add Meta Data Value for the fabric with the index
		/// @param fabricIndex: target Fabric 
		/// @param metaDataKey: target Key
		/// @param metaDataValue: the new value for the key
		virtual void ChangeMetaDataValueForFabric(unsigned int fabricIndex, const std::string& metaDataKey, const std::string& metaDataValue) {}

		/// @fn ChangeMetaDataValueForFabric(unsigned int fabricIndex, const wstring& metaDataKey, const wstring& metaDataValue)
		/// @brief Change or add Meta Data Value for the fabric with the index
		/// @param fabricIndex: target Fabric 
		/// @param metaDataKey: target Key
		/// @param metaDataValue: the new value for the key
		virtual void ChangeMetaDataValueForFabricW(unsigned int fabricIndex, const std::wstring& metaDataKey, const std::wstring& metaDataValue) {}

		/// @fn GetFabricSize()
		/// @brief Get the number of fabrics on the Object Browser
		/// @return return the number of fabrics on the object browser
		virtual int GetFabricSize() { return 0; } // deprecated. please use 'GetFabricCount()' function

		/// @fn GetFabricIndexForPattern(int patternIndex)
		/// @brief Get the fabric index which is assigned on the pattern with the pattern index
		/// @param patternIndex: the input pattern index to get the fabric index
		/// @return fabric index on the object browser for the pattern with the pattern index
		virtual int GetFabricIndexForPattern(int patternIndex) { return 0; }

		/// @fn GetFabricIndex(const string& fabricName)
		/// @brief Get the fabric index which is using the fabric name
		/// @param fabricName: the fabric name to get the fabric index
		/// @return fabric index
		virtual int GetFabricIndex(const std::string& fabricName) { return 0; }

		/// @fn GetFabricIndexW(const wstring& fabricName)
		/// @brief Get the fabric index which is using the fabric name
		/// @param fabricName: the fabric name to get the fabric index
		/// @return fabric index
		virtual int GetFabricIndexW(const std::wstring& fabricName) { return 0; }

		/// @fn GetFabricName(int fabricIndex)
		/// @brief Get the fabric name of the fabric in the fabricIndex on the object browser
		/// @parm fabricIndex: the fabric index to get the name
		/// @return fabric name
		virtual std::string GetFabricName(int fabricIndex) { return ""; }

		/// @fn GetFabricNameW(int fabricIndex)
		/// @brief Get the fabric name of the fabric in the fabricIndex on the object browser
		/// @parm fabricIndex: the fabric index to get the name
		/// @return fabric name
		virtual std::wstring GetFabricNameW(int fabricIndex) { return L""; }

		/// @fn GetColorwayFabricInfo(int colorwayIndex, int fabricIndex)
		/// @brief Get a fabric information for a colorway
		/// @param colorwayIndex: the colorway index for the fabric to get the fabric info
		/// @param fabricIndex: the fabricIndex for the fabric to get the fabric info
		/// @return json string for fabric information
		virtual std::string GetColorwayFabricInfo(int colorwayIndex, int fabricIndex) { return ""; }

		/// @fn GetColorwayFabricInfoW(int colorwayIndex, int fabricIndex)
		/// @brief Get a fabric information for a colorway
		/// @param colorwayIndex: the colorway index for the fabric to get the fabric info
		/// @param fabricIndex: the fabricIndex for the fabric to get the fabric info
		/// @return json string for fabric information
		virtual std::wstring GetColorwayFabricInfoW(int colorwayIndex, int fabricIndex) { return L""; }

		/// @fn GetAPIMetaDataFromFile(const string& filePath)
		/// @brief Get API meta data for the fabric
		/// @param filePath: filepath (.zfab / .jfab)
		/// @return json string for meta data [key - value] list
		virtual std::string GetAPIMetaDataFromFile(const std::string& filePath) { return ""; }

		/// @fn GetAPIMetaDataFromFileW(const wstring& filePath)
		/// @brief Get API meta data for the fabric
		/// @param filePath: filepath (.zfab / .jfab)
		/// @return json string for meta data [key - value] list
		virtual std::wstring GetAPIMetaDataFromFileW(const std::wstring& filePath) { return L""; }

		/// @fn SetFabricInformation(int _fabricIndex, const std::map<string, string> _infoMap)
		/// @brief Set Fabric Information to fabric
		/// @param _fabricIndex: the fabricIndex for the fabric to set the fabric info
		/// @param _infoMap: Fabric Information (etc. Classification, Content, SupplierName, Owner)
		virtual void SetFabricInformation(int _fabricIndex, const std::map<std::string, std::string> _infoMap) {}

		/// @fn SetFabricInformationW(int _fabricIndex, const std::map<wstring, wstring> _infoMap)
		/// @brief Set Fabric Information to fabric
		/// @param _fabricIndex: the fabricIndex for the fabric to set the fabric info
		/// @param _infoMap: Fabric Information (etc. Classification, Content, SupplierName, Owner)
		virtual void SetFabricInformationW(int _fabricIndex, const std::map<std::wstring, std::wstring>& _infoMap) {}

		/// @fn GetFabricInformation(int _fabricIndex)
		/// @brief Get Fabric Information for fabric
		/// @param _fabricIndex: the fabricIndex for the fabric to get the fabric info
		/// @return Fabric Information
		virtual std::map<std::string, std::string> GetFabricInformation(int _fabricIndex)
		{
			std::map<std::string, std::string> infoMap;
			return infoMap;
		}

		/// @fn GetFabricInformationW(int _fabricIndex)
		/// @brief Get Fabric Information for fabric
		/// @param _fabricIndex: the fabricIndex for the fabric to get the fabric info
		/// @return Fabric Information
		virtual std::map<std::wstring, std::wstring> GetFabricInformationW(int _fabricIndex)
		{
			std::map<std::wstring, std::wstring> infoMap;
			return infoMap;
		}

		/// @fn SetCustomImage(int _fabricIndex, const string& _filePath)
		/// @brief Set Custom Image to fabric
		/// @param _fabricIndex: the fabricIndex for the fabric
		/// @param _filePath: filepath  (.png / .jpeg)
		virtual void SetCustomImage(int _fabricIndex, const std::string& _filePath) {}

		/// @fn SetCustomImageW(int _fabricIndex, const wstring& _filePath)
		/// @brief Set Custom Image to fabric
		/// @param _fabricIndex: the fabricIndex for the fabric
		/// @param _filePath: filepath  (.png / .jpeg)
		virtual void SetCustomImageW(int _fabricIndex, const std::wstring& _filePath) {}

		/// @fn GetPrimaryFabric()
		/// @brief Return the fabric index used for a major number of the patterns
		virtual int GetPrimaryFabric() { return -1; }

		/// @fn SetFabricName(unsigned int index, const string& str)
		/// @brief Change fabric name
		/// @param index: the target fabric index to change the name
		/// @param str: new name for the fabric
		virtual void SetFabricName(unsigned int index, const std::string& str) {}

		/// @fn SetColorwayNameW(unsigned int index, const wstring& wstr)
		/// @brief Change fabric name
		/// @param index: the target fabric index to change the name
		/// @param str: new name for the fabric
		virtual void SetFabricNameW(unsigned int index, const std::wstring& wstr) {}

		/// @fn GetFabricStyleNameList()
		/// @brief Return all fabric style name
		virtual std::vector<std::string> GetFabricStyleNameList()
		{
			std::vector<std::string> fabricStyleList;
			return fabricStyleList;
		}

		/// @fn ImportSubstanceFile(unsigned int fabricIndex, const string& substanceFilePath)
		/// @brief Import substance to a fabric. 
		/// @parm fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm substanceFilePath: the substance file path to import
		/// @return if it succeeds, return true.
		virtual bool ImportSubstanceFile(unsigned int fabricIndex, const std::string& substanceFilePath) { return false; }

		/// @fn ImportSubstanceFileW(unsigned int fabricIndex, const wstring& substanceFilePath)
		/// @brief Import substance to a fabric. 
		/// @parm fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm substanceFilePath: the substance file path to import
		/// @return if it succeeds, return true.
		virtual bool ImportSubstanceFileW(unsigned int fabricIndex, const std::wstring& substanceFilePath) { return false; }

		/// @fn GetFabricWidth(int fabricIndex)
		/// @brief Get Fabric width (mm)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @return fabric width
		virtual float GetFabricWidth(int fabricIndex) { return 0.f;  }

		/// @fn SetFabricWidth(int fabricIndex, float mm)
		/// @brief Set Fabric width (mm)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @parm mm: Fabric width
		/// @return void
		virtual void SetFabricWidth(int fabricIndex, float mm) {}

		/// @fn GetFabricLength(int fabricIndex)
		/// @brief Get Fabric Length (mm)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @return Fabric Length
		virtual float GetFabricLength(int fabricIndex) { return 0.f; }

		/// @fn SetTextureMapping(unsigned int fabricIndex, int materialFace, int mappingType)
		/// @brief Set Texture Mapping Type (Repeat or Unified)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @parm mapping type: the target mapping type to set. 0: "Repeat", 1: "Unified"
		/// @return void
		virtual void SetTextureMapping(unsigned int fabricIndex, int mappingType) {}

		/// @fn SetSubstancePreset(unsigned int fabricIndex, int materialFace, int preset)
		/// @brief Set Texture Mapping Type (Repeat or Unified)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @parm materialFace: the target material index in the property editor. 0: "Front", 1: "Back", 2: "Side"
		/// @parm presetIndex: the preset index to set
		/// @return void
		virtual void SetSubstancePreset(unsigned int fabricIndex, int materialFace, int presetIndex) {}

		/// @fn SetSubstanceResolution(unsigned int fabricIndex, int materialFace, int resolution)
		/// @brief Set Texture Mapping Type (Repeat or Unified)
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @parm materialFace: the target material index in the property editor. 0: "Front", 1: "Back", 2: "Side"
		/// @parm resolutionIndex: the resolution index to set
		/// @return void
		virtual void SetSubstanceResolution(unsigned int fabricIndex, int materialFace, int resolutionIndex) {}

		/// @fn ImportSubstanceFileAsFaceType(unsigned int _fabricIndex, const string& _substanceFilePath, const string& _faceType)
		/// @brief Import substance to a fabric as facetype. 
		/// @parm _fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm _substanceFilePath: the substance file path to import
		/// @parm _facetype :“Front”, “Back”, “Side” 
		/// @return if it succeeds, return true.
		virtual bool ImportSubstanceFileAsFaceType(unsigned int _fabricIndex, const std::string& _substanceFilePath, const std::string& _faceType) { return false; }

		/// @fn ImportSubstanceFileAsFaceTypeW(unsigned int _fabricIndex, const wstring& _substanceFilePath, const string& _faceType)
		/// @brief Import substance to a fabric. 
		/// @parm _fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm _substanceFilePath: the substance file path to import
		/// @parm _facetype :“Front”, “Back”, “Side” 
		/// @return if it succeeds, return true.
		virtual bool ImportSubstanceFileAsFaceTypeW(unsigned int _fabricIndex, const std::wstring& _substanceFilePath, const std::string& _faceType) { return false; }

		/// @fn SetUseSameMaterialAsFront(unsigned int fabricIndex, int materialFace, bool _useSameMaterial)
		/// @brief Set use same material as front. 
		/// @parm fabricIndex: the target fabric index in the object browser
		/// @parm materialFace: the target material index in the property editor. 1: "Back", 2: "Side"
		/// @parm _useSameMaterial: use same material as front
		/// @return void
		virtual void SetUseSameMaterialAsFront(unsigned int fabricIndex, int materialFace, bool _useSameMaterial) {}

		/*
		Set a base color of PBR material in a fabric using given RGBA values.
		@return if it succeeds, return true.
		*/
		/// @fn SetFabricPBRMaterialBaseColor(unsigned int colorwayIndex, unsigned int fabricIndex, unsigned int materialFace, float r, float g, float b, float a)
		/// @brief Set a base color of PBR material in a fabric using given RGBA values.
		/// @parm colorwayIndex: colorway index
		/// @parm fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm materialFace :“Front”, “Back”, “Side” 
		/// @parm r : To be set as red value
		/// @parm g : To be set as green value
		/// @parm b : To be set as blue value
		/// @parm a : To be set as alpha value
		/// @return if it succeeds, return true.
		virtual bool SetFabricPBRMaterialBaseColor(unsigned int colorwayIndex, unsigned int fabricIndex, unsigned int materialFace, float r, float g, float b, float a) = 0;


		/// @fn GetFabricPBRMaterialBaseColor(unsigned int colorwayIndex, unsigned int fabricIndex, unsigned int materialFace)
		/// @brief Get a base color of PBR material in a fabric.
		/// @parm colorwayIndex: colorway index
		/// @parm fabricIndex: the target fabric index in the object browser to import substance file onto
		/// @parm materialFace :“Front”, “Back”, “Side” 
		/// @return RGBA float tuple
		virtual std::tuple<float, float, float, float> GetFabricPBRMaterialBaseColor(unsigned int colorwayIndex, unsigned int fabricIndex, unsigned int materialFace) = 0;

		/// @fn SetCurrentFabricIndex(int _index)
		/// @brief Set Current Fabric Index
		/// @parm _index: Fabric Index
		/// @return Fabric setting success or failure return
		virtual bool SetCurrentFabricIndex(int _index) { return true; };

		/// @fn CreateZfabFromTextures(const std::string& _filePath, const std::string& _baseTexturePath, const std::string& _normalTexturePath, const std::string& _disPlacementTexturePath, const std::string& _opacityTexturePath)
		/// @brief Create Zfab From Textures
		/// @param _filePath: created filepath (.zfab)
		/// @parm _baseTexturePath: the baseTexture file path to create zfab
		/// @parm _normalTexturePath: the normalTexturePath file path to create zfab
		/// @parm _disPlacementTexturePath :the disPlacementTexturePath file path to create zfab
		/// @parm _opacityTexturePath :the opacityTexturePath file path to create zfab
		/// @return zfab file creating success or failure return
		virtual bool CreateZfabFromTextures(const std::string& _filePath, const std::string& _baseTexturePath, const std::string& _normalTexturePath, const std::string& _disPlacementTexturePath, const std::string& _opacityTexturePath) { return false; }

		/// @fn CombineZfab(const std::string& _filePath, const std::string& _baseZfabPath, const std::string& _targetZfabPath)
		/// @brief Combining two zfab files into one
		/// @param _filePath: created filepath (.zfab)
		/// @parm _baseZfabPath: the Path to the zfab file on which the physical properties will be based
		/// @parm _targetZfabPath: the path to the zfab file on which the image will be based
		virtual void CombineZfab(const std::string& _filePath, const std::string& _baseZfabPath, const std::string& _targetZfabPath) {}


		/// @fn int GetRoughnessType()
		/// @brief Get Material RoughnessType
		/// @parm _index: Fabric Index, _materialFace : Fabric Material Face Type (0 : Front , 1 : BACK , 2 : SIDE)
		/// @return Return fabric's mateiral Roughness Type. ( 0 : Intensity, 1 : Map )
		virtual int GetRoughnessType(int _index, int _materialFace) = 0;

		/// @fn SetRoughnessType(int _index, int _materialFace, int _roughnessType)
		/// @brief Set Current Fabric's mateiral roughnessType
		/// @parm _index: Fabric Index, _materialFace : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE), _roughnessType : roughnessType to be set ( 0 : Intensity, 1 : Map )
		/// @return Fabric setting success or failure return
		virtual void SetRoughnessType(int _index, int _materialFace, int _roughnessType) = 0;

		/// @fn float GetRoughnessValueIntensity(int _index, int _face)
		/// @brief Get fabric material's Roughness Intensity values.
		/// @parm _index: Fabric Index, _face : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE)
		/// @return Fabric material's Roughness Intensity values.
		virtual int GetRoughnessValueIntensity(int _index, int _face) = 0;

		/// @fn float GetRoughnessValueMapIntensity(int _index, int _face)
		/// @brief Get fabric material's map type roughness intensity values.
		/// @parm _index: Fabric Index, _face : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE)
		/// @return Fabric material's map type roughness Intensity values.
		virtual int GetRoughnessValueMapIntensity(int _index, int _face) = 0;

		/// @fn SetRoughnessValueMapIntensity(int _index, int _face, int _value)
		/// @brief Set fabric material's map type roughness intensity values.
		/// @parm _index: Fabric Index, _face : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE), _value : intensity value to be set.
		virtual void SetRoughnessValueMapIntensity(int _index, int _face, int _value) = 0;

		/// @fn bool IsRoughnessValueMapInvert(int _index, int _face)
		/// @brief Return whether roughness map is inverted.
		/// @parm _index: Fabric Index, _face : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE)
		/// @Return whether roughness map is inverted.
		virtual bool IsRoughnessValueMapInvert(int _index, int _face) = 0;

		/// @fn bool SetRoughnessValueMapInvert(int _index, int _face, bool _bInvert)
		/// @brief SetS whether roughness map is inverted
		/// @parm _index: Fabric Index, _face : target material Face Type (0 : Front , 1 : BACK , 2 : SIDE), bInvert : Whether roughness map is inverted.
		virtual void SetRoughnessValueMapInvert(int _index, int _face, bool _bInvert)  = 0;
	};
}
